/* main.c - Application main entry point */

/*
 * Copyright (c) 2015-2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/types.h>
#include <stddef.h>
#include <zephyr/sys/printk.h>
#include <zephyr/sys/util.h>

#include <zephyr/bluetooth/bluetooth.h>
#include <zephyr/bluetooth/hci.h>

#include <zephyr/bluetooth/hci.h>
#include <zephyr/bluetooth/hci_vs.h>

static int set_bd_addr(void)
{
	int err = 0;
	struct net_buf *buf;
	struct net_buf *rsp;
	struct bt_hci_cp_vs_write_bd_addr *cmd_params;

	buf = bt_hci_cmd_create(BT_HCI_OP_VS_WRITE_BD_ADDR, sizeof(*cmd_params));
	if (!buf) {
		printk("Could not allocate command buffer\n");
		return -ENOMEM;
	}

	cmd_params = net_buf_add(buf, sizeof(*cmd_params));

	cmd_params->bdaddr.val[0]= 0xCC;
	cmd_params->bdaddr.val[1]= 0xCC;
	cmd_params->bdaddr.val[2]= 0xCC;
	cmd_params->bdaddr.val[3]= 0xCC;
	cmd_params->bdaddr.val[4]= 0xCC;
	cmd_params->bdaddr.val[5]= 0xCC;

	printk("bt_hci_cmd_send_sync \n");

	err = bt_hci_cmd_send_sync(BT_HCI_OP_VS_WRITE_BD_ADDR, buf, &rsp);
	if (err)
	{
		printk("err: %d \n", err);
		return err;
	}

	printk("Successfully set bd addr \n");

	printk("\nResponse: ");
	for (int i = 0; i < rsp->len; i++)
	{
		printk("%.2X ", rsp->data[i]);
	}
	printk("\n\n");
	net_buf_unref(rsp);

	return 0;
}

#define DEVICE_NAME CONFIG_BT_DEVICE_NAME
#define DEVICE_NAME_LEN (sizeof(DEVICE_NAME) - 1)

/*
 * Set Advertisement data. Based on the Eddystone specification:
 * https://github.com/google/eddystone/blob/master/protocol-specification.md
 * https://github.com/google/eddystone/tree/master/eddystone-url
 */
static const struct bt_data ad[] = {
	BT_DATA_BYTES(BT_DATA_FLAGS, BT_LE_AD_NO_BREDR),
	BT_DATA_BYTES(BT_DATA_UUID16_ALL, 0xaa, 0xfe),
	BT_DATA_BYTES(BT_DATA_SVC_DATA16,
		      0xaa, 0xfe, /* Eddystone UUID */
		      0x10, /* Eddystone-URL frame type */
		      0x00, /* Calibrated Tx power at 0m */
		      0x00, /* URL Scheme Prefix http://www. */
		      'z', 'e', 'p', 'h', 'y', 'r',
		      'p', 'r', 'o', 'j', 'e', 'c', 't',
		      0x08) /* .org */
};

/* Set Scan Response data */
static const struct bt_data sd[] = {
	BT_DATA(BT_DATA_NAME_COMPLETE, DEVICE_NAME, DEVICE_NAME_LEN),
};

static void start_advertising(void)
{
	char addr_s[BT_ADDR_LE_STR_LEN];

	/* Start advertising */
	int err = bt_le_adv_start(
		BT_LE_ADV_PARAM(
			BT_LE_ADV_OPT_CONNECTABLE | BT_LE_ADV_OPT_USE_IDENTITY,
			BT_GAP_ADV_FAST_INT_MIN_2,
			BT_GAP_ADV_FAST_INT_MAX_2,
			NULL),
		ad,
		ARRAY_SIZE(ad),
		sd,
		ARRAY_SIZE(sd)
		);
	if (err) {
		printk("Advertising failed to start (err %d)\n", err);
		return;
	}

	/* For connectable advertising you would use
	 * bt_le_oob_get_local().  For non-connectable non-identity
	 * advertising an non-resolvable private address is used;
	 * there is no API to retrieve that.
	 */
	struct bt_le_oob oob_info = {0};
	bt_le_oob_get_local(BT_ID_DEFAULT, &oob_info);
	bt_addr_le_to_str(&oob_info.addr, addr_s, sizeof(addr_s));

	printk("Beacon started, advertising as %s\n", addr_s);
}

int main(void)
{
	int err;

	printk("Starting Beacon Demo\n");

	/* Initialize the Bluetooth Subsystem */
	err = bt_enable(NULL);
	if (err) {
		printk("Bluetooth init failed (err %d)\n", err);
		return err;
	}

	set_bd_addr();

	start_advertising();
	return 0;
}
